/*
 *  Queue.h
 *  Prototype
 *
 *  Created by Stefan Dlugolinsky on 24.2.2008.
 *  Copyright 2008 Hviezdna jedenastka. All rights reserved.
 *
 */

#ifndef QUEUE_H_
#define QUEUE_H_

#include <assert.h>

template<class T>
class Queue {

private:
	int maxsize;
	int count;
	int head;
	int tail;
	T* data;
	
public:
	Queue(int _maxsize);
	~Queue();

	bool empty();
	T& front();
	void pop();
	void push(const T& element);
	int size();
};

template<class T>
Queue<T>::Queue(int _maxsize) {

	assert(_maxsize > 0);

	maxsize = _maxsize;
	count = 0;
	head = tail = -1;
	data = new T[maxsize];
};

template<class T>
Queue<T>::~Queue() {

	delete[] data;
};

template<class T>
bool Queue<T>::empty() {

	return head == -1;
};

template<class T>
T& Queue<T>::front() {

	assert(!empty());
	
	return data[head];
};

template<class T>
void Queue<T>::pop() {

	if (!empty()) {
			
		if (head == tail) {
		
			head = tail = -1;
			
		} else {
	
			head = (head + 1) % maxsize;
		}
		
		count--;
	}
};

template<class T>
void Queue<T>::push(const T& element) {

	if (empty()) {
	
		head = tail = 0;
		
	} else {

		tail = (tail + 1) % maxsize;
	
		if (tail == head) {
		
			head = (head + 1) % maxsize;
			count--;
		}
	}
	
	data[tail] = element;
	count++;
};

template<class T>
int Queue<T>::size() {

	return count;
};

#endif /*QUEUE_H_*/